@@ -9,27 +9,33 @@
 ## VanderWeele 2015 p468 Proposition 2.4
 ##' Create calculators for effects and se (mreg linear / yreg logistic)
 ##'
-##' Construct functions for the conditional effect estimates and their standard errors in the mreg linear / yreg logistic setting. Internally, this function deconstruct model objects and feed parameter estiamtes to the internal worker functions \code{calc_myreg_mreg_linear_yreg_logistic_est} and \code{calc_myreg_mreg_linear_yreg_logistic_se}.
+##' Construct functions for the conditional effect estimates and their standard errors in the mreg linear / yreg logistic setting. Internally, this function deconstructs model objects and feeds parameter estimates to the internal worker functions \code{calc_myreg_mreg_linear_yreg_logistic_est} and \code{calc_myreg_mreg_linear_yreg_logistic_se}.
 ##'
 ##' @inheritParams regmedint
 ##' @param mreg_fit Model fit from \code{\link{fit_mreg}}
 ##' @param yreg_fit Model fit from \code{\link{fit_yreg}}
 ##'
-##' @return A list contraining a function for effect estimates and a function for corresponding standard errors.
+##' @return A list containing a function for effect estimates and a function for corresponding standard errors.
 calc_myreg_mreg_linear_yreg_logistic <- function(mreg,
                                                  mreg_fit,
                                                  yreg,
                                                  yreg_fit,
                                                  avar,
                                                  mvar,
-                                                 cvar, # This can be NULL.
+                                                 cvar, 
+                                                 emm_ac_mreg,
+                                                 emm_ac_yreg,
+                                                 emm_mc_yreg,
                                                  interaction) {
+  
+  ### 
 
     ## mreg coefficients
     beta_hat <- beta_hat_helper(mreg = mreg,
                                 mreg_fit = mreg_fit,
                                 avar = avar,
-                                cvar = cvar)
+                                cvar = cvar,
+                                emm_ac_mreg = emm_ac_mreg)
     ## This mreg linear yreg logistic is the only case that uses sigma^2.
     sigma_sq_hat <- sigma_hat_sq(mreg_fit = mreg_fit)
     ## yreg coefficients
@@ -38,6 +44,8 @@ calc_myreg_mreg_linear_yreg_logistic <- function(mreg,
                                   avar = avar,
                                   mvar = mvar,
                                   cvar = cvar,
+                                  emm_ac_yreg = emm_ac_yreg,
+                                  emm_mc_yreg = emm_mc_yreg,
                                   interaction = interaction)
     ## Construct a function of (a1, a0, m_cde, c_cond) that returns
     ## a vector of point estimates for quantities of interest.
@@ -45,23 +53,29 @@ calc_myreg_mreg_linear_yreg_logistic <- function(mreg,
         calc_myreg_mreg_linear_yreg_logistic_est(beta0 = beta_hat$beta0,
                                                  beta1 = beta_hat$beta1,
                                                  beta2 = beta_hat$beta2,
+                                                 beta3 = beta_hat$beta3,
                                                  theta0 = theta_hat$theta0,
                                                  theta1 = theta_hat$theta1,
                                                  theta2 = theta_hat$theta2,
                                                  theta3 = theta_hat$theta3,
                                                  theta4 = theta_hat$theta4,
+                                                 theta5 = theta_hat$theta5,
+                                                 theta6 = theta_hat$theta6,
                                                  sigma_sq = sigma_sq_hat)
 
     ## vcovs
     Sigma_beta_hat <- Sigma_beta_hat(mreg = mreg,
                                      mreg_fit = mreg_fit,
                                      avar = avar,
-                                     cvar = cvar)
+                                     cvar = cvar,
+                                     emm_ac_mreg = emm_ac_mreg)
     Sigma_theta_hat <- Sigma_theta_hat(yreg = yreg,
                                        yreg_fit = yreg_fit,
                                        avar = avar,
                                        mvar = mvar,
                                        cvar = cvar,
+                                       emm_ac_yreg = emm_ac_yreg,
+                                       emm_mc_yreg = emm_mc_yreg,
                                        interaction = interaction)
     Sigma_sigma_sq_hat <- Sigma_sigma_sq_hat(mreg_fit = mreg_fit)
     ## Construct a function of (a0, a1, m_cde, c_cond) that returns
@@ -70,11 +84,14 @@ calc_myreg_mreg_linear_yreg_logistic <- function(mreg,
         calc_myreg_mreg_linear_yreg_logistic_se(beta0 = beta_hat$beta0,
                                                 beta1 = beta_hat$beta1,
                                                 beta2 = beta_hat$beta2,
+                                                beta3 = beta_hat$beta3,
                                                 theta0 = theta_hat$theta0,
                                                 theta1 = theta_hat$theta1,
                                                 theta2 = theta_hat$theta2,
                                                 theta3 = theta_hat$theta3,
                                                 theta4 = theta_hat$theta4,
+                                                theta5 = theta_hat$theta5,
+                                                theta6 = theta_hat$theta6,
                                                 sigma_sq = sigma_sq_hat,
                                                 Sigma_beta = Sigma_beta_hat,
                                                 Sigma_theta = Sigma_theta_hat,
@@ -94,21 +111,27 @@ calc_myreg_mreg_linear_yreg_logistic <- function(mreg,
 calc_myreg_mreg_linear_yreg_logistic_est <- function(beta0,
                                                      beta1,
                                                      beta2,
+                                                     beta3,
                                                      theta0,
                                                      theta1,
                                                      theta2,
                                                      theta3,
                                                      theta4,
+                                                     theta5,
+                                                     theta6,
                                                      sigma_sq) {
 
     validate_myreg_coefs(beta0 = beta0,
                          beta1 = beta1,
                          beta2 = beta2,
+                         beta3 = beta3,
                          theta0 = theta0,
                          theta1 = theta1,
                          theta2 = theta2,
                          theta3 = theta3,
                          theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6,
                          sigma_sq = sigma_sq)
 
     ## Construct a function for point estimates given (a0, a1, m_cde, c_cond).
@@ -124,30 +147,58 @@ calc_myreg_mreg_linear_yreg_logistic_est <- function(beta0,
             assertthat::assert_that(length(c_cond) == length(beta2))
             beta2_c <- sum(t(matrix(beta2)) %*% matrix(c_cond))
         }
+        
+        if (is.null(beta3)) {
+            beta3_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(beta3))
+            beta3_c <- sum(t(matrix(beta3)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta4)) {
+            assertthat::assert_that(is.null(c_cond))
+            theta4_c <- 0
+        } else {
+            assertthat::assert_that(!is.null(c_cond))
+            assertthat::assert_that(length(c_cond) == length(theta4))
+            theta4_c <- sum(t(matrix(theta4)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta5)) {
+            theta5_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta5))
+            theta5_c <- sum(t(matrix(theta5)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta6)) {
+            theta6_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta6))
+            theta6_c <- sum(t(matrix(theta6)) %*% matrix(c_cond))
+        }
 
-        ## VanderWeele 2015 p468
+        ## Extension of VanderWeele 2015 p468. Estimates are all on log OR scale
         ## Adopted from mediation.sas and modified.
         ## Look up the third occurrence of the following:
         ## %if &yreg^=linear & &mreg=linear & &interaction=true %then %do;
-        cde <- (theta1 + (theta3 * m_cde)) * (a1 - a0)
+        cde <- (theta1 + theta3 * m_cde + theta5_c) * (a1 - a0)
         ## Pearl decomposition (Regular NDE and NIE)
-        ## Note the a0 in the first line.                      vv
-        pnde <- (theta1 + (theta3 * beta0) + (theta3 * beta1 * a0) +
-                 (theta3 * beta2_c) + (theta3 * theta2 * sigma_sq)) * (a1 - a0) +
-            ((1/2) * theta3^2 * sigma_sq) * (a1^2 - a0^2)
-        ## Note the a1.                              vv
-        tnie <- ((theta2 * beta1) + theta3 * beta1 * a1) * (a1 - a0)
+        ## Note the a0 in the first line.                      
+        pnde <- ((theta1 + theta5_c + theta3*(beta0 + beta1*a0 + beta2_c + beta3_c*a0) +
+                 1/2*sigma_sq*theta3*(2*theta2 + theta3*a1 + theta3*a0 + 2*theta6_c))) * (a1 - a0)
+        ## Note the a1.                              
+        tnie <- (theta2 + theta3*a1 + theta6_c) * (beta1 + beta3_c) * (a1 - a0)
         ## Another decomposition
-        ## Note the a0 -> a1 change in the first line.         vv
-        tnde <- (theta1 + (theta3 * beta0) + (theta3 * beta1 * a1) +
-                 (theta3 * beta2_c) + (theta3 * theta2 * sigma_sq)) * (a1 - a0) +
-            ((1/2) * theta3^2 * sigma_sq) * (a1^2 - a0^2)
-        ## Note the a1 -> a0 change.                 vv
-        pnie <- ((theta2 * beta1) + theta3 * beta1 * a0) * (a1 - a0)
+        ## Note the a0 -> a1 change in the first line.         
+        tnde <- ((theta1 + theta5_c + theta3*(beta0 + beta1*a1 + beta2_c + beta3_c*a1) +
+                      1/2*sigma_sq*theta3*(2*theta2 + theta3*a1 + theta3*a0 + 2*theta6_c))) * (a1 - a0)
+        ## Note the a1 -> a0 change.                 
+        pnie <- (theta2 + theta3*a0 + theta6_c) * (beta1 + beta3_c) * (a1 - a0)
         ## It is the sum of NDE and NIE on the log scale.
         te <- pnde + tnie
         ## VanderWeele 2015 p48.
-        pm <- (exp(pnde) * (exp(tnie) - 1)) / (exp(te) - 1)
+        pm <- exp(pnde) * (exp(tnie) - 1) / (exp(te) - 1)
 
         ## Return a named vector
         c(cde  = unname(cde),
@@ -166,11 +217,14 @@ calc_myreg_mreg_linear_yreg_logistic_est <- function(beta0,
 calc_myreg_mreg_linear_yreg_logistic_se <- function(beta0,
                                                     beta1,
                                                     beta2,
+                                                    beta3,
                                                     theta0,
                                                     theta1,
                                                     theta2,
                                                     theta3,
                                                     theta4,
+                                                    theta5,
+                                                    theta6,
                                                     sigma_sq,
                                                     Sigma_beta,
                                                     Sigma_theta,
@@ -179,21 +233,27 @@ calc_myreg_mreg_linear_yreg_logistic_se <- function(beta0,
     validate_myreg_coefs(beta0 = beta0,
                          beta1 = beta1,
                          beta2 = beta2,
+                         beta3 = beta3,
                          theta0 = theta0,
                          theta1 = theta1,
                          theta2 = theta2,
                          theta3 = theta3,
                          theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6,
                          sigma_sq = sigma_sq)
 
     validate_myreg_vcovs(beta0 = beta0,
                          beta1 = beta1,
                          beta2 = beta2,
+                         beta3 = beta3,
                          theta0 = theta0,
                          theta1 = theta1,
                          theta2 = theta2,
                          theta3 = theta3,
                          theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6,
                          sigma_sq = sigma_sq,
                          Sigma_beta = Sigma_beta,
                          Sigma_theta = Sigma_theta,
@@ -216,79 +276,181 @@ calc_myreg_mreg_linear_yreg_logistic_se <- function(beta0,
             assertthat::assert_that(length(c_cond) == length(beta2))
             beta2_c <- sum(t(matrix(beta2)) %*% matrix(c_cond))
         }
+        
+        if (is.null(beta3)) {
+            beta3_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(beta3))
+            beta3_c <- sum(t(matrix(beta3)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta4)) {
+            assertthat::assert_that(is.null(c_cond))
+            theta4_c <- 0
+        } else {
+            assertthat::assert_that(!is.null(c_cond))
+            assertthat::assert_that(length(c_cond) == length(theta4))
+            theta4_c <- sum(t(matrix(theta4)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta5)) {
+            theta5_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta5))
+            theta5_c <- sum(t(matrix(theta5)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta6)) {
+            theta6_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta6))
+            theta6_c <- sum(t(matrix(theta6)) %*% matrix(c_cond))
+        }
 
         ## VanderWeele 2015. p468
         ## Valeri & VanderWeele 2013. Appendix p6-9
         ## These are the gradient vector of each scalar quantity of interest.
         ## Obtain the first partial derivative wrt to each parameter.
+        if(is.null(theta5)){
+            pd_cde_theta5 <- rep(0, length(theta5))
+        }else{
+            pd_cde_theta5 <- c_cond
+        }
         Gamma_cde <-
             matrix(c(0,                       # beta0
                      0,                       # beta1
                      rep(0, length(beta2)),   # beta2 vector
+                     rep(0, length(beta3)),   # beta3 vector
                      ##
                      0,                       # theta0
                      1,                       # theta1
                      0,                       # theta2
                      m_cde,                   # theta3
                      rep(0, length(theta4)),  # theta4 vector
+                     pd_cde_theta5,           # theta5 vector
+                     rep(0, length(theta6)),  # theta6 vector
                      ##
                      0))                      # sigma^2
+        
         ##
+        if(is.null(beta3)){
+            pd_pnde_beta3 <- rep(0, length(beta3))
+        }else{
+            pd_pnde_beta3 <- theta3*a1*c_cond
+        }
+        if(is.null(theta5)){
+            pd_pnde_theta5 <- rep(0, length(theta5))
+        }else{
+            pd_pnde_theta5 <- c_cond
+        }
+        if(is.null(theta6)){
+            pd_pnde_theta6 <- rep(0, length(theta6))
+        }else{
+            pd_pnde_theta6 <- theta3*sigma_sq*c_cond
+        }
         Gamma_pnde <-
             matrix(c(
                 theta3,                                             # beta0
-                (theta3 * a0),                                      # beta1
-                (theta3 * c_cond),                                  # beta2 vector
+                theta3*a0,                                          # beta1
+                theta3*c_cond,                                      # beta2 vector
+                pd_pnde_beta3,                                      # beta3 vector
                 ##
                 0,                                                  # theta0
                 1,                                                  # theta1
-                (theta3 * sigma_sq),                                # theta2
-                (beta0 + beta1 * a0 + beta2_c + theta2 *
-                 sigma_sq + theta3 * sigma_sq * (a1 + a0)),         # theta3
+                theta3*sigma_sq,                                    # theta2
+                a0*(beta1+beta3_c) + beta0 + beta2_c + sigma_sq*((theta2+theta6_c) + theta3*(a0+a1)),  # theta3
                 rep(0, length(theta4)),                             # theta4 vector
+                pd_pnde_theta5,                                     # theta5 vector
+                pd_pnde_theta6,                                     # theta6 vector
                 ##
-                (theta3 * theta2 + (1/2) * theta3^2 * (a1 + a0))))  # sigma^2
+                1/2*theta3*(2*(theta2+theta6_c) + theta3*(a0+a1))))  # sigma^2
+        
         ##
+        if(is.null(beta3)){
+            pd_tnie_beta3 <- rep(0, length(beta3))
+        }else{
+            pd_tnie_beta3 <- c_cond*(theta2 + theta3*a1 + theta6_c)
+        }
+        if(is.null(theta6)){
+            pd_tnie_theta6 <- rep(0, length(theta6))
+        }else{
+            pd_tnie_theta6 <- c_cond*(beta1 + beta3_c)
+        }
         Gamma_tnie <-
             matrix(c(
                 0,                       # beta0
-                (theta2 + theta3 * a1),  # beta1
+                theta2 + theta3*a1 + theta6_c,  # beta1
                 rep(0, length(beta2)),   # beta2 vector
+                pd_tnie_beta3,           # beta3 vector
                 ##
                 0,                       # theta0
                 0,                       # theta1
-                beta1,                   # theta2
-                (beta1 * a1),            # theta3
+                beta1 + beta3_c,         # theta2
+                a1*(beta1 + beta3_c),    # theta3
                 rep(0, length(theta4)),  # theta4 vector
+                rep(0, length(theta5)),  # theta5 vector
+                pd_tnie_theta6,          # theta6 vector
                 ##
                 0))                      # sigma^2
+        
         ##
+        if(is.null(beta3)){
+            pd_tnde_beta3 <- rep(0, length(beta3))
+        }else{
+            pd_tnde_beta3 <- theta3*a1*c_cond
+        }
+        if(is.null(theta5)){
+            pd_tnde_theta5 <- rep(0, length(theta5))
+        }else{
+            pd_tnde_theta5 <- c_cond
+        }
+        if(is.null(theta6)){
+            pd_tnde_theta6 <- rep(0, length(theta6))
+        }else{
+            pd_tnde_theta6 <- theta3*sigma_sq*c_cond
+        }
         Gamma_tnde <-
             matrix(c(
-                theta3,                                             # beta0
-                (theta3 * a1),                                      # beta1 a0 -> a1
-                (theta3 * c_cond),                                  # beta2 vector
+                theta3,                                           # beta0
+                theta3*a1,                                        # beta1 a0 -> a1
+                theta3*c_cond,                                    # beta2 vector
+                pd_tnde_beta3,                                    # beta3 vector
                 ##
-                0,                                                  # theta0
-                1,                                                  # theta1
-                (theta3 * sigma_sq),                                # theta2
-                (beta0 + beta1 * a1 + beta2_c + theta2 *
-                 sigma_sq + theta3 * sigma_sq * (a1 + a0)),         # theta3 a0 -> a1
-                rep(0, length(theta4)),                             # theta4 vector
+                0,                                                # theta0
+                1,                                                # theta1
+                theta3*sigma_sq,                                  # theta2
+                a1*(beta1+beta3_c) + beta0 + beta2_c + sigma_sq*((theta2+theta6_c) + theta3*(a0+a1)),   # theta3 a0 -> a1
+                rep(0, length(theta4)),                           # theta4 vector
+                pd_tnde_theta5,                                   # theta5 vector
+                pd_tnde_theta6,                                   # theta6 vector
                 ##
-                (theta3 * theta2 + (1/2) * theta3^2 * (a1 + a0))))  # sigma^2
+                1/2*theta3*(2*(theta2+theta6_c) + theta3*(a0+a1))))  # sigma^2
+       
         ##
+        if(is.null(beta3)){
+            pd_pnie_beta3 <- rep(0, length(beta3))
+        }else{
+            pd_pnie_beta3 <- c_cond * (theta2 + theta3*a0 + theta6_c)
+        }
+        if(is.null(theta6)){
+            pd_pnie_theta6 <- rep(0, length(theta6))
+        }else{
+            pd_pnie_theta6 <- c_cond*(beta1 + beta3_c)
+        }
         Gamma_pnie <-
             matrix(c(
                 0,                       # beta0
-                (theta2 + theta3 * a0),  # beta1 a1 -> a0
+                theta2 + theta3*a0 + theta6_c,  # beta1 a1 -> a0
                 rep(0, length(beta2)),   # beta2 vector
+                pd_pnie_beta3,           # beta3 vector
                 ##
                 0,                       # theta0
                 0,                       # theta1
-                beta1,                   # theta2
-                (beta1 * a0),            # theta3 a1 -> a0
+                beta1 + beta3_c,         # theta2
+                a0*(beta1 + beta3_c),    # theta3
                 rep(0, length(theta4)),  # theta4 vector
+                rep(0, length(theta5)),  # theta5 vector
+                pd_pnie_theta6,          # theta6 vector
                 ##
                 0))                      # sigma^2
         ## Note VV2013 Appendix p9 seems to have an error in the gradient
@@ -300,10 +462,9 @@ calc_myreg_mreg_linear_yreg_logistic_se <- function(beta0,
         ## Not implemented in mediation.sas.
         ## Not mentioned in VV2013, VV2015, or VanderWeele 2015.
         ## Copied from calc_myreg_mreg_linear_yreg_logistic_est
-        pnde <- (theta1 + (theta3 * beta0) + (theta3 * beta1 * a0) +
-                 (theta3 * beta2_c) + (theta3 * theta2 * sigma_sq)) * (a1 - a0) +
-            ((1/2) * theta3^2 * sigma_sq) * (a1^2 - a0^2)
-        tnie <- ((theta2 * beta1) + theta3 * beta1 * a1) * (a1 - a0)
+        pnde <- ((theta1 + theta5_c + theta3*(beta0 + beta1*a0 + beta2_c + beta3_c*a0) +
+                      1/2*sigma_sq*theta3*(2*theta2 + theta3*a1 + theta3*a0 + 2*theta6_c))) * (a1 - a0)
+        tnie <- (theta2 + theta3*a1 + theta6_c) * (beta1 + beta3_c) * (a1 - a0)
         ## Gradient of pm wrt pnde and tnie. A vector of two.
         ## Need to unname argument vectors to get c(pnde = , tnie = ).
         d_pm <- grad_prop_med_yreg_logistic(pnde = unname(pnde), tnie = unname(tnie))
@@ -314,7 +475,7 @@ calc_myreg_mreg_linear_yreg_logistic_se <- function(beta0,
         ##                   (d_pm / d_tnie) * (d_tnie / d_params)
         ## where (d_pnde / d_params) is (a1 - a0) * Gamma_pnde and
         ##       (d_tnie / d_params) is (a1 - a0) * Gamma_tnie.
-        ## FIXME: This is not tested aginst a reference standard.
+        ## FIXME: This is not tested against a reference standard.
         Gamma_pm <- d_pm[["pnde"]] * Gamma_pnde + d_pm[["tnie"]] * Gamma_tnie
 
 
